/* global jest describe beforeEach it expect */

'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _underscore = require('underscore');

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

jest.dontMock('../components/ComboBox');
jest.dontMock('../components/Option');

var TestUtils = _reactAddons2['default'].addons.TestUtils;

// Babel would move an import in front of the jest.dontMock. That's why require
// is used instead of import.
var ComboBox = require('../components/ComboBox');
var Option = require('../components/Option');

describe('ComboBox', function () {
  it('should initialise inputValue & filteredOptions during construction', function () {
    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { value: 'vie' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    expect(combobox.state.inputValue).toBe('vie');
    expect(_reactAddons2['default'].Children.count(combobox.filteredOptions)).toBe(1);
  });

  it('should filter all values case no value, defaultValue or valueLink is defined', function () {
    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      null,
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    expect(combobox.state.inputValue).toBe(undefined);
    expect(_reactAddons2['default'].Children.count(combobox.filteredOptions)).toBe(2);
  });

  it('should be able to provide a onUpdate callback', function () {
    var wasCalled = false;

    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { onUpdate: function () {
          wasCalled = true;
        } },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(combobox, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(wasCalled).toBeTruthy();
  });

  it('should provide a parameter with 2 fields: value and identifier in onUpdate callback', function () {
    var value = undefined;
    var identifier = undefined;
    var isMatch = undefined;
    var isSelect = undefined;

    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { onUpdate: function (obj) {
          value = obj.value;identifier = obj.identifier;isMatch = obj.isMatchingOption;isSelect = obj.isOptionSelection;
        } },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome', identifier: 1 },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', identifier: 2, className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(combobox, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(value).toBe('vienna');
    expect(identifier).toBe(2);
    expect(isMatch).toBe(true);
    expect(isSelect).toBe(true);
  });

  it('should be able to provide a valueLink', function () {
    // let wasCalled = false;

    var valueLink = {
      requestChange: function requestChange() {
        // wasCalled = true;
      },
      value: 'vie'
    };

    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { valueLink: valueLink },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna123', className: 'vienna-option' },
        'Vienna'
      )
    ));

    expect(combobox.filteredOptions.length).toBe(2);

    var viennaOptionNode = TestUtils.scryRenderedDOMComponentsWithClass(combobox, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    // strangely line below is failing
    // expect(wasCalled).toBeTruthy();
  });

  it('should change the inputValue on selection', function () {
    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { defaultValue: 'vie' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna123' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(combobox, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(combobox.state.inputValue).toBe('vienna');
  });

  it('should be able to adopt the styles of a combobox', function () {
    var combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      ComboBox,
      { style: { cursor: 'cross' } },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    var selectedAreaNode = TestUtils.scryRenderedDOMComponentsWithTag(combobox, 'input')[1];
    expect(selectedAreaNode.props.style.cursor).toBe('cross');
  });

  describe('updating props', function () {
    var combobox = undefined;

    beforeEach(function () {
      combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
        ComboBox,
        null,
        _reactAddons2['default'].createElement(
          Option,
          { value: 'rome' },
          'Rome'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'vienna' },
          'Vienna'
        )
      ));
    });

    it('should update it\'s state in case value is provided', function () {
      var properties = (0, _underscore.extend)({}, combobox.props, { value: 'vienna' });
      combobox.componentWillReceiveProps(properties);

      expect(combobox.state.inputValue).toBe('vienna');
    });

    it('should update it\'s state in case value is provided', function () {
      var valueLink = {
        requestChange: function requestChange() {},
        value: 'vienna'
      };

      var properties = (0, _underscore.extend)({}, combobox.props, { valueLink: valueLink });
      combobox.componentWillReceiveProps(properties);

      expect(combobox.state.inputValue).toBe('vienna');
    });

    /*
     it('should not update it\'s state in case defaultValue is updated', () => {
     const properties = extend({}, combobox.props, { defaultValue: "vienna" });
     combobox.componentWillReceiveProps(properties);
      expect(combobox.state.inputValue).toBe("rome");
     });
     */
  });

  function testKeyEvents(container) {
    it('should open the menu by pressing ArrowDown', function () {
      TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowDown' });
      expect(container.combobox.state.isOpen).toBeTruthy();
    });

    it('should open the menu by pressing ArrowUp', function () {
      TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowUp' });
      expect(container.combobox.state.isOpen).toBeTruthy();
    });

    it('should open the menu by pressing Space', function () {
      TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowUp' });
      expect(container.combobox.state.isOpen).toBeTruthy();
    });

    describe('when the menu is open', function () {
      beforeEach(function () {
        container.combobox.setState({ isOpen: true });
      });

      it('should close menu when pressing Escape', function () {
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'Escape' });
        expect(container.combobox.state.isOpen).toBeFalsy();
      });

      it('should focus on the next option when pressing ArrowDown', function () {
        container.combobox.setState({ focusedOptionIndex: 0 });
        expect(container.combobox.state.focusedOptionIndex).toBe(0);
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowDown' });
        expect(container.combobox.state.focusedOptionIndex).toBe(1);
      });

      it('should focus on the first option when pressing ArrowDown and none was focused on', function () {
        container.combobox.setState({ focusedOptionIndex: undefined });
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowDown' });
        expect(container.combobox.state.focusedOptionIndex).toBe(0);
      });

      it('should focus on the previous option when pressing ArrowUp', function () {
        container.combobox.setState({ focusedOptionIndex: 2 });
        expect(container.combobox.state.focusedOptionIndex).toBe(2);
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowUp' });
        expect(container.combobox.state.focusedOptionIndex).toBe(1);
      });

      it('should focus on the last option when pressing ArrowUp and none was focused on', function () {
        container.combobox.setState({ focusedOptionIndex: undefined });
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'ArrowUp' });
        expect(container.combobox.state.focusedOptionIndex).toBe(2);
      });

      it('should select the focused option when pressing Enter', function () {
        container.combobox.setState({ focusedOptionIndex: 2 });
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'Enter' });
        expect(container.combobox.state.inputValue).toBe('berlin');
      });

      it('should select the focused option when pressing Tab', function () {
        container.combobox.setState({ focusedOptionIndex: 2 });
        TestUtils.Simulate.keyDown(container.comboNode, { key: 'Tab' });
        expect(container.combobox.state.inputValue).toBe('berlin');
      });
    });
  }

  describe('manage key events for simple list', function () {
    // in order to ensure no references are lost a container object is used
    var container = {};

    beforeEach(function () {
      container.combobox = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
        ComboBox,
        null,
        _reactAddons2['default'].createElement(
          Option,
          { value: 'rome' },
          'Rome'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'vienna' },
          'Vienna'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'berlin' },
          'Berlin'
        )
      ));
      container.comboNode = TestUtils.scryRenderedDOMComponentsWithTag(container.combobox, 'input')[1];
    });

    testKeyEvents(container);
  });
});