/* global jest describe beforeEach it expect */

'use strict';

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _underscore = require('underscore');

var _reactAddons = require('react/addons');

var _reactAddons2 = _interopRequireDefault(_reactAddons);

jest.dontMock('../components/Select');
jest.dontMock('../components/Option');
jest.dontMock('../components/Placeholder');
jest.dontMock('../components/Separator');

var TestUtils = _reactAddons2['default'].addons.TestUtils;

// Babel would move an import in front of the jest.dontMock. That's why require
// is used instead of import.
var Select = require('../components/Select');
var Option = require('../components/Option');
var Placeholder = require('../components/Placeholder');
var Separator = require('../components/Separator');

describe('Select', function () {
  it('should initialise selectedValue & focusedOptionValue during construction', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { value: 'vienna' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    expect(select.state.selectedValue).toBe('vienna');
    expect(select.state.focusedOptionValue).toBe('vienna');
  });

  it('should take the first option in case no value, defaultValue or valueLink is defined', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    expect(select.state.selectedValue).toBe('rome');
    expect(select.state.focusedOptionValue).toBe('rome');
  });

  it('should not have any option selected, but focusedOn the first in case there is a Placeholder & no value, defaultValue or valueLink is defined', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Placeholder,
        null,
        'Select a City'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    expect(select.state.selectedValue).toBeUndefined();
    expect(select.state.focusedOptionValue).toBe('rome');
  });

  it('should render the content of selected option', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { value: 'vienna' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    var selectedOptionArea = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];
    expect(selectedOptionArea.props.children[0]._store.props.children).toBe('Vienna');
  });

  it('should render the placeholder content', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Placeholder,
        null,
        'Select a City'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    var selectedOptionArea = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];

    expect(selectedOptionArea.props.children[0]._store.props.children).toBe('Select a City');
  });

  it('should work with one option provided', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      )
    ));

    var selectedOptionArea = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];
    expect(selectedOptionArea.props.children[0]._store.props.children).toBe('Rome');
  });

  it('should work with a placehoder and an array of options provided', function () {
    var options = [{ value: 'rome', content: 'Rome' }, { value: 'vienna', content: 'Vienna' }];

    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Placeholder,
        null,
        'Select a City'
      ),
      options.map(function (option, index) {
        return _reactAddons2['default'].createElement(
          Option,
          { key: index, value: option.value },
          option.content
        );
      })
    ));

    var selectedOptionArea = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];
    var entries = TestUtils.scryRenderedDOMComponentsWithTag(select, 'li');

    expect(selectedOptionArea.props.children[0]._store.props.children).toBe('Select a City');
    expect(entries[0].props.children._store.props.children).toBe('Rome');
    expect(entries[1].props.children._store.props.children).toBe('Vienna');
  });

  it('should work with a placehoder and a combination of single options and an array of options provided', function () {
    var options = [{ value: 'rome', content: 'Rome' }, { value: 'vienna', content: 'Vienna' }];

    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      null,
      _reactAddons2['default'].createElement(
        Placeholder,
        null,
        'Select a City'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'boston' },
        'Boston'
      ),
      options.map(function (option, index) {
        return _reactAddons2['default'].createElement(
          Option,
          { key: index, value: option.value },
          option.content
        );
      }),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'newyork' },
        'New York'
      )
    ));

    var selectedOptionArea = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];
    var entries = TestUtils.scryRenderedDOMComponentsWithTag(select, 'li');

    expect(selectedOptionArea.props.children[0]._store.props.children).toBe('Select a City');
    expect(entries[0].props.children._store.props.children).toBe('Boston');
    expect(entries[1].props.children._store.props.children).toBe('Rome');
    expect(entries[2].props.children._store.props.children).toBe('Vienna');
    expect(entries[3].props.children._store.props.children).toBe('New York');
  });

  it('should be able to provide a valueLink', function () {
    var wasCalled = false;

    var valueLink = {
      requestChange: function requestChange() {
        wasCalled = true;
      },
      value: 'rome'
    };

    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { valueLink: valueLink },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(select, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(wasCalled).toBeTruthy();
  });

  it('should be able to provide a onUpdate callback', function () {
    var wasCalled = false;

    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { onUpdate: function () {
          wasCalled = true;
        } },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(select, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(wasCalled).toBeTruthy();
  });

  it('should change the selectedValue & focusedOptionValue on selection', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { defaultValue: 'rome' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(select, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(select.state.selectedValue).toBe('vienna');
    expect(select.state.focusedOptionValue).toBe('vienna');
  });

  it('should not change the selectedValue & focusedOptionValue on selection in case props.value is provided', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { value: 'rome' },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna', className: 'vienna-option' },
        'Vienna'
      )
    ));

    var viennaOptionNode = TestUtils.findRenderedDOMComponentWithClass(select, 'vienna-option');
    TestUtils.Simulate.click(viennaOptionNode);

    expect(select.state.selectedValue).toBe('rome');
    expect(select.state.focusedOptionValue).toBe('rome');
  });

  it('should be able to adopt the styles of a select', function () {
    var select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
      Select,
      { style: { cursor: 'cross' } },
      _reactAddons2['default'].createElement(
        Option,
        { value: 'rome' },
        'Rome'
      ),
      _reactAddons2['default'].createElement(
        Option,
        { value: 'vienna' },
        'Vienna'
      )
    ));

    var selectedAreaNode = TestUtils.scryRenderedDOMComponentsWithTag(select, 'div')[1];
    expect(selectedAreaNode.props.style.cursor).toBe('cross');
  });

  describe('updating props', function () {
    var select = undefined;

    beforeEach(function () {
      select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
        Select,
        null,
        _reactAddons2['default'].createElement(
          Option,
          { value: 'rome' },
          'Rome'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'vienna' },
          'Vienna'
        )
      ));
    });

    it('should update it\'s state in case value is provided', function () {
      var properties = (0, _underscore.extend)({}, select.props, { value: 'vienna' });
      select.componentWillReceiveProps(properties);

      expect(select.state.selectedValue).toBe('vienna');
    });

    it('should update it\'s state in case value is provided', function () {
      var valueLink = {
        requestChange: function requestChange() {},
        value: 'vienna'
      };

      var properties = (0, _underscore.extend)({}, select.props, { valueLink: valueLink });
      select.componentWillReceiveProps(properties);

      expect(select.state.selectedValue).toBe('vienna');
    });

    it('should not update it\'s state in case defaultValue is updated', function () {
      var properties = (0, _underscore.extend)({}, select.props, { defaultValue: 'vienna' });
      select.componentWillReceiveProps(properties);

      expect(select.state.selectedValue).toBe('rome');
    });
  });

  function testKeyEvents(container) {
    it('should open the menu by pressing ArrowDown', function () {
      TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowDown' });
      expect(container.select.state.isOpen).toBeTruthy();
    });

    it('should open the menu by pressing ArrowUp', function () {
      TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowUp' });
      expect(container.select.state.isOpen).toBeTruthy();
    });

    it('should open the menu by pressing Space', function () {
      TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowUp' });
      expect(container.select.state.isOpen).toBeTruthy();
    });

    describe('when the menu is open', function () {
      beforeEach(function () {
        container.select.setState({ isOpen: true });
      });

      it('should close menu when pressing Escape', function () {
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'Escape' });
        expect(container.select.state.isOpen).toBeFalsy();
      });

      it('should focus on the next option when pressing ArrowDown', function () {
        expect(container.select.state.focusedOptionValue).toBe('rome');
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowDown' });
        expect(container.select.state.focusedOptionValue).toBe('vienna');
      });

      it('should focus on the first option when pressing ArrowDown and none was focused on', function () {
        container.select.setState({ focusedOptionValue: undefined });
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowDown' });
        expect(container.select.state.focusedOptionValue).toBe('rome');
      });

      it('should focus on the previous option when pressing ArrowUp', function () {
        container.select.setState({ focusedOptionValue: 'vienna' });
        expect(container.select.state.focusedOptionValue).toBe('vienna');
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowUp' });
        expect(container.select.state.focusedOptionValue).toBe('rome');
      });

      it('should focus on the last option when pressing ArrowUp and none was focused on', function () {
        container.select.setState({ focusedOptionValue: undefined });
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'ArrowUp' });
        expect(container.select.state.focusedOptionValue).toBe('berlin');
      });

      it('should select the focused option when pressing Enter', function () {
        container.select.setState({ focusedOptionValue: 'berlin' });
        TestUtils.Simulate.keyDown(container.selectNode, { key: 'Enter' });
        expect(container.select.state.selectedValue).toBe('berlin');
      });

      it('should select the focused option when pressing Space', function () {
        container.select.setState({ focusedOptionValue: 'berlin' });
        TestUtils.Simulate.keyDown(container.selectNode, { key: ' ' });
        expect(container.select.state.selectedValue).toBe('berlin');
      });
    });
  }

  describe('manage key events for simple list', function () {
    // in order to ensure no references are lost a container object is used
    var container = {};

    beforeEach(function () {
      container.select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
        Select,
        null,
        _reactAddons2['default'].createElement(
          Option,
          { value: 'rome' },
          'Rome'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'vienna' },
          'Vienna'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'berlin' },
          'Berlin'
        )
      ));
      container.selectNode = _reactAddons2['default'].findDOMNode(container.select);
    });

    testKeyEvents(container);
  });

  describe('manage key events when separators are present', function () {
    // in order to ensure no references are lost a container object is used
    var container = {};

    beforeEach(function () {
      container.select = TestUtils.renderIntoDocument(_reactAddons2['default'].createElement(
        Select,
        null,
        _reactAddons2['default'].createElement(
          Separator,
          null,
          'Italy'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'rome' },
          'Rome'
        ),
        _reactAddons2['default'].createElement(
          Separator,
          null,
          'Austria'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'vienna' },
          'Vienna'
        ),
        _reactAddons2['default'].createElement(
          Separator,
          null,
          'Germany'
        ),
        _reactAddons2['default'].createElement(
          Option,
          { value: 'berlin' },
          'Berlin'
        )
      ));
      container.selectNode = _reactAddons2['default'].findDOMNode(container.select);
    });

    testKeyEvents(container);
  });
});